@@ -48,18 +48,12 @@ from IPython.display import display, HTML, SVG
 
 import pyAgrum as gum
 from pyAgrum.lib.bn2graph import BN2dot, BNinference2dot
+from pyAgrum.lib.cn2graph import CN2dot, CNinference2dot
 from pyAgrum.lib.id2graph import ID2dot, LIMIDinference2dot
 from pyAgrum.lib.mn2graph import MN2UGdot, MNinference2UGdot
 from pyAgrum.lib.mn2graph import MN2FactorGraphdot, MNinference2FactorGraphdot
 from pyAgrum.lib.bn_vs_bn import GraphicalBNComparator
-from pyAgrum.lib.proba_histogram import proba2histo
-
-# check if an instance of ipython exists
-try:
-  get_ipython
-except NameError as e:
-  raise ImportError(
-    "[pyAgrum ERROR] pyAgrum.lib.notebook has to be import from an IPython's instance (mainly notebook).") from None
+from pyAgrum.lib.proba_histogram import proba2histo, probaMinMaxH
 
 _cdict = {
   'red': ((0.0, 0.1, 0.3),
@@ -156,7 +150,7 @@ def _reprGraph(gr, size, asString):
   else:
     i = Image(format="png", data=gr.create_png())
     if asString:
-      return '<img style="margin:0" src="data:image/png;base64,{}"/>'.format(base64.encodebytes(i.data).decode())
+      return '<img style="margin:0" src="data:image/png;base64,{}"/>'.format(encodebytes(i.data).decode())
     else:
       display_png(i)
 
@@ -311,10 +305,23 @@ def showProba(p, scale=1.0):
   Show a mono-dim Potential
 
   :param p: the mono-dim Potential
-  :return:
+  :param scale: the scale (zoom)
   """
   fig = proba2histo(p, scale)
-  #  fig.patch.set_facecolor(gum.config["notebook", "figure_facecolor"])
+  IPython.display.set_matplotlib_formats(
+    gum.config["notebook", "graph_format"])
+  plt.show()
+
+
+def showProbaMinMax(pmin, pmax, scale=1.0):
+  """
+  Show a bi-Potential (min,max
+
+  :param pmin: the mono-dim Potential for min values
+  :param pmin: the mono-dim Potential for max values
+  :param scale: the scale (zoom)
+  """
+  fig = probaMinMaxH(pmin, pmax, scale)
   IPython.display.set_matplotlib_formats(
     gum.config["notebook", "graph_format"])
   plt.show()
@@ -477,6 +484,28 @@ def showBN(bn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=Non
   return showGraph(BN2dot(bn, size, nodeColor, arcWidth, arcColor, cmap, cmapArc), size)
 
 
+def showCN(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=None, cmapArc=None):
+  """
+  show a credal network
+
+  :param cn: the credal network
+  :param size: size of the rendered graph
+  :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
+  :param arcWidth: a arcMap of values to be shown as width of arcs
+  :param arcColor: a arcMap of values (between 0 and 1) to be shown as color of arcs
+  :param cmap: color map to show the colors
+  :param cmapArc: color map to show the arc color if distinction is needed
+  :return: the graph
+  """
+  if size is None:
+    size = gum.config["notebook", "default_graph_size"]
+
+  if cmapArc is None:
+    cmapArc = cmap
+
+  return showGraph(CN2dot(cn, size, nodeColor, arcWidth, arcColor, cmap, cmapArc), size)
+
+
 def getMN(mn, view=None, size=None, nodeColor=None, factorColor=None, edgeWidth=None, edgeColor=None, cmap=None,
           cmapEdge=None):
   """
@@ -534,6 +563,29 @@ def getBN(bn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=None
   return getGraph(BN2dot(bn, size, nodeColor, arcWidth, arcColor, cmap, cmapArc), size)
 
 
+def getCN(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmap=None, cmapArc=None):
+  """
+  get a HTML string for a credal network
+
+  :param cn: the credal network
+  :param size: size of the rendered graph
+  :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
+  :param arcWidth: a arcMap of values to be shown as width of arcs
+  :param arcColor: a arcMap of values (between 0 and 1) to be shown as color of arcs
+  :param cmap: color map to show the colors
+  :param cmapArc: color map to show the arc color if distinction is needed
+
+  :return: the graph
+  """
+  if size is None:
+    size = gum.config["notebook", "default_graph_size"]
+
+  if cmapArc is None:
+    cmapArc = cmap
+
+  return getGraph(CN2dot(cn, size, nodeColor, arcWidth, arcColor, cmap, cmapArc), size)
+
+
 def _normalizeVals(vals, hilightExtrema=False):
   """
   normalisation if vals is not a proba (max>1)
@@ -682,8 +734,13 @@ def _get_showInference(model, engine=None, evs=None, targets=None, size=None,
     if engine is None:
       engine = gum.ShaferShenoyLIMIDInference(model)
     return LIMIDinference2dot(model, size, engine, evs, targets)
+  elif isinstance(model, gum.CredalNet):
+    if engine is None:
+      engine = gum.CNMonteCarloSampling(model)
+    return CNinference2dot(model, size, engine, evs, targets, nodeColor, arcWidth, arcColor, cmap)
   else:
-    raise gum.InvalidArgument("Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
+    raise gum.InvalidArgument(
+      "Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
 
 
 def showInference(model, engine=None, evs=None, targets=None, size=None,
@@ -1035,37 +1092,62 @@ def getInferenceEngine(ie, inferenceCaption):
 
 def getJT(jt, size=None):
   if gum.config["notebook", "junctiontree_with_names"] == "True":
-    cliqlabels = lambda c: " ".join(sorted([model.variable(n).name() for n in jt.clique(c)]))
-    cliqnames = lambda c: "-".join(sorted([model.variable(n).name() for n in jt.clique(c)]))
-    seplabels = lambda c1, c2: " ".join(sorted([model.variable(n).name() for n in jt.separator(c1, c2)]))
-    sepnames = lambda c1, c2: cliqnames(c1) + '+' + cliqnames(c2)
+    def cliqlabels(c):
+      return " ".join(
+        sorted([model.variable(n).name() for n in jt.clique(c)]))
+
+    def cliqnames(
+        c):
+      return "-".join(sorted([model.variable(n).name() for n in jt.clique(c)]))
+
+    def seplabels(c1, c2):
+      return " ".join(
+        sorted([model.variable(n).name() for n in jt.separator(c1, c2)]))
+
+    def sepnames(c1, c2):
+      return cliqnames(c1) + '+' + cliqnames(c2)
   else:
-    cliqlabels = lambda c: " ".join([str(n) for n in sorted(jt.clique(c))])
-    cliqnames = lambda c: "-".join([str(n) for n in sorted(jt.clique(c))])
-    seplabels = lambda c1, c2: " ".join([str(n) for n in sorted(jt.separator(c1, c2))])
-    sepnames = lambda c1, c2: cliqnames(c1) + '^' + cliqnames(c2)
+    def cliqlabels(c):
+      return " ".join([str(n) for n in sorted(jt.clique(c))])
+
+    def cliqnames(c):
+      return "-".join([str(n) for n in sorted(jt.clique(c))])
+
+    def seplabels(c1, c2):
+      return " ".join(
+        [str(n) for n in sorted(jt.separator(c1, c2))])
+
+    def sepnames(c1, c2):
+      return cliqnames(c1) + '^' + cliqnames(c2)
 
   model = jt._engine._model
-  name = model.propertyWithDefault("name", str(type(model)).split(".")[-1][:-2])
+  name = model.propertyWithDefault(
+    "name", str(type(model)).split(".")[-1][:-2])
   graph = dot.Dot(graph_type='graph', graph_name=name, bgcolor="transparent")
   for c in jt.nodes():
     graph.add_node(dot.Node('"' + cliqnames(c) + '"',
                             label='"' + cliqlabels(c) + '"',
                             style="filled",
-                            fillcolor=gum.config["notebook", "junctiontree_clique_bgcolor"],
-                            fontcolor=gum.config["notebook", "junctiontree_clique_fgcolor"],
+                            fillcolor=gum.config["notebook",
+                                                 "junctiontree_clique_bgcolor"],
+                            fontcolor=gum.config["notebook",
+                                                 "junctiontree_clique_fgcolor"],
                             fontsize=gum.config["notebook", "junctiontree_clique_fontsize"]))
   for c1, c2 in jt.edges():
     graph.add_node(dot.Node('"' + sepnames(c1, c2) + '"',
                             label='"' + seplabels(c1, c2) + '"',
                             style="filled",
                             shape="box", width="0", height="0", margin="0.02",
-                            fillcolor=gum.config["notebook", "junctiontree_separator_bgcolor"],
-                            fontcolor=gum.config["notebook", "junctiontree_separator_fgcolor"],
+                            fillcolor=gum.config["notebook",
+                                                 "junctiontree_separator_bgcolor"],
+                            fontcolor=gum.config["notebook",
+                                                 "junctiontree_separator_fgcolor"],
                             fontsize=gum.config["notebook", "junctiontree_separator_fontsize"]))
   for c1, c2 in jt.edges():
-    graph.add_edge(dot.Edge('"' + cliqnames(c1) + '"', '"' + sepnames(c1, c2) + '"'))
-    graph.add_edge(dot.Edge('"' + sepnames(c1, c2) + '"', '"' + cliqnames(c2) + '"'))
+    graph.add_edge(dot.Edge('"' + cliqnames(c1) +
+                            '"', '"' + sepnames(c1, c2) + '"'))
+    graph.add_edge(dot.Edge('"' + sepnames(c1, c2) +
+                            '"', '"' + cliqnames(c2) + '"'))
 
   if size is None:
     size = gum.config["notebook", "default_graph_size"]
@@ -1088,30 +1170,161 @@ def getCliqueGraph(cg, size=None):
     return getDot(cg.toDot())
 
 
-# hook to control some parameters for notebook
+def show(model, size=None):
+  """
+  propose a (visual) representation of a model in a notebook
+
+  :param GraphicalModel model: the model to show (pyAgrum.BayesNet, pyAgrum.MarkovNet, pyAgrum.InfluenceDiagram or pyAgrum.Potential)
+
+  :param int size: optional size for the graphical model (no effect for Potential)
+  """
+  if isinstance(model, gum.BayesNet):
+    showBN(model, size)
+  elif isinstance(model, gum.MarkovNet):
+    showMN(model, size)
+  elif isinstance(model, gum.InfluenceDiagram):
+    showInfluenceDiagram(model, size)
+  elif isinstance(model, gum.CredalNet):
+    showCN(model, size)
+  elif isinstance(model, gum.Potential):
+    showPotential(model)
+  else:
+    raise gum.InvalidArgument(
+      "Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
+
+
+def show(model, **kwargs):
+  """
+  propose a (visual) representation of a model in a notebook
+
+  :param GraphicalModel model: the model to show (pyAgrum.BayesNet, pyAgrum.MarkovNet, pyAgrum.InfluenceDiagram or pyAgrum.Potential)
+
+  :param int size: optional size for the graphical model (no effect for Potential)
+  """
+  if isinstance(model, gum.BayesNet):
+    showBN(model, **kwargs)
+  elif isinstance(model, gum.MarkovNet):
+    showMN(model, **kwargs)
+  elif isinstance(model, gum.InfluenceDiagram):
+    showInfluenceDiagram(model, **kwargs)
+  elif isinstance(model, gum.CredalNet):
+    showCN(model, **kwargs)
+  elif isinstance(model, gum.Potential):
+    showPotential(model)
+  elif hasattr(model, "toDot"):
+    showDot(dot.graph_from_dot_data(model.toDot()), **kwargs)
+  else:
+    raise gum.InvalidArgument(
+      "Argument model should be a PGM (BayesNet, MarkovNet, Influence Diagram or Potential or ...")
+
+
+def export(model, filename, **kwargs):
+  """
+  export the graphical representation of the model in filename (png, pdf,etc.)
+
+  :param filename : the file to write
+
+  :param GraphicalModel model: the model to show (pyAgrum.BayesNet, pyAgrum.MarkovNet, pyAgrum.InfluenceDiagram or pyAgrum.Potential)
+
+  :param int size: optional size for the graphical model (no effect for Potential)
+  """
+  format = filename.split(".")[-1]
+  if format not in ['pdf', 'png', 'fig', 'jpg', 'svg','ps']:
+    raise Exception(
+      f"{filename} in not a correct filename for export : extension '{format}' not in [pdf,png,fig,jpg,svg].")
+
+  if isinstance(model, gum.BayesNet):
+    fig = BN2dot(model, **kwargs)
+  elif isinstance(model, gum.MarkovNet):
+    if gum.config["notebook", "default_markovnetwork_view"] == "graph":
+      fig = MN2UGdot(model, **kwargs)
+    else:
+      fig = MN2FactorGraphdot(model, **kwargs)
+  elif isinstance(model, gum.InfluenceDiagram):
+    fig = ID2dot(model, **kwargs)
+  elif isinstance(model, gum.CredalNet):
+    fig = CN2dot(model, **kwargs)
+  elif hasattr(model, "toDot"):
+    fig = dot.graph_from_dot_data(model.toDot(), **kwargs)
+  else:
+    raise gum.InvalidArgument(
+      "Argument model should be a PGM (BayesNet, MarkovNet or Influence Diagram")
+  fig.write(filename, format=format)
+
+
+def exportInference(model, filename, engine=None, evs=None, targets=None, size=None,
+                    nodeColor=None, factorColor=None,
+                    arcWidth=None, arcColor=None,
+                    cmap=None, cmapArc=None, graph=None, view=None):
+  """
+  the graphical representation of an inference in a notebook
+
+  :param GraphicalModel model: the model in which to infer (pyAgrum.BayesNet, pyAgrum.MarkovNet or
+          pyAgrum.InfluenceDiagram)
+  :param gum.Inference engine: inference algorithm used. If None, gum.LazyPropagation will be used for BayesNet,
+          gum.ShaferShenoy for gum.MarkovNet and gum.ShaferShenoyLIMIDInference for gum.InfluenceDiagram.
+  :param dictionnary evs: map of evidence
+  :param set targets: set of targets
+  :param string size: size of the rendered graph
+  :param nodeColor: a nodeMap of values (between 0 and 1) to be shown as color of nodes (with special colors for 0 and 1)
+  :param factorColor: a nodeMap of values (between 0 and 1) to be shown as color of factors (in MarkovNet representation)
+  :param arcWidth: a arcMap of values to be shown as width of arcs
+  :param arcColor: a arcMap of values (between 0 and 1) to be shown as color of arcs
+  :param cmap: color map to show the color of nodes and arcs
+  :param cmapArc: color map to show the vals of Arcs.
+  :param graph: only shows nodes that have their id in the graph (and not in the whole BN)
+  :param view: graph | factorgraph | None (default) for Markov network
+  :return: the desired representation of the inference
+
+  warning:: In order to export probability distribution in nodes, exportInference needs that Graphviz includes the Cairo renderer
+  """
+  format = filename.split(".")[-1]
+  if format not in ['pdf', 'png', 'fig', 'jpg', 'svg']:
+    raise Exception(
+      f"{filename} in not a correct filename for export : extension '{format}' not in [pdf,png,fig,jpg,svg].")
+
+  fig = _get_showInference(model, engine, evs, targets, size, nodeColor, arcWidth, arcColor, cmap, cmapArc, graph,
+                           view)
+  fig.write(filename, format=format)
+
+
 def _update_config():
+  # hook to control some parameters for notebook when config changes
   mpl.rcParams['figure.facecolor'] = gum.config["notebook", "figure_facecolor"]
-  IPython.display.set_matplotlib_formats(gum.config["notebook", "graph_format"])
+  IPython.display.set_matplotlib_formats(
+    gum.config["notebook", "graph_format"])
 
 
-gum.config.add_hook(_update_config)
-gum.config.run_hooks()
+# check if an instance of ipython exists
+try:
+  get_ipython
+except NameError as e:
+  import warnings
 
-# adding _repr_html_ to some pyAgrum classes !
-gum.BayesNet._repr_html_ = lambda self: getBN(self)
-gum.MarkovNet._repr_html_ = lambda self: getMN(self)
-gum.BayesNetFragment._repr_html_ = lambda self: getBN(self)
-gum.InfluenceDiagram._repr_html_ = lambda self: getInfluenceDiagram(self)
+  warnings.warn("""
+  ** pyAgrum.lib.notebook has to be import from an IPython's instance (mainly notebook).
+  """)
+else:
+  gum.config.add_hook(_update_config)
+  gum.config.run_hooks()
 
-gum.CliqueGraph._repr_html_ = lambda self: getCliqueGraph(self)
+  # adding _repr_html_ to some pyAgrum classes !
+  gum.BayesNet._repr_html_ = lambda self: getBN(self)
+  gum.BayesNetFragment._repr_html_ = lambda self: getBN(self)
+  gum.MarkovNet._repr_html_ = lambda self: getMN(self)
+  gum.BayesNetFragment._repr_html_ = lambda self: getBN(self)
+  gum.InfluenceDiagram._repr_html_ = lambda self: getInfluenceDiagram(self)
+  gum.CredalNet._repr_html_ = lambda self: getCN(self)
 
-gum.Potential._repr_html_ = lambda self: getPotential(self)
-gum.LazyPropagation._repr_html_ = lambda self: getInferenceEngine(
-  self, "Lazy Propagation on this BN")
+  gum.CliqueGraph._repr_html_ = lambda self: getCliqueGraph(self)
 
-gum.UndiGraph._repr_html_ = lambda self: getDot(self.toDot())
-gum.DiGraph._repr_html_ = lambda self: getDot(self.toDot())
-gum.MixedGraph._repr_html_ = lambda self: getDot(self.toDot())
-gum.DAG._repr_html_ = lambda self: getDot(self.toDot())
-gum.EssentialGraph._repr_html_ = lambda self: getDot(self.toDot())
-gum.MarkovBlanket._repr_html_ = lambda self: getDot(self.toDot())
+  gum.Potential._repr_html_ = lambda self: getPotential(self)
+  gum.LazyPropagation._repr_html_ = lambda self: getInferenceEngine(
+    self, "Lazy Propagation on this BN")
+
+  gum.UndiGraph._repr_html_ = lambda self: getDot(self.toDot())
+  gum.DiGraph._repr_html_ = lambda self: getDot(self.toDot())
+  gum.MixedGraph._repr_html_ = lambda self: getDot(self.toDot())
+  gum.DAG._repr_html_ = lambda self: getDot(self.toDot())
+  gum.EssentialGraph._repr_html_ = lambda self: getDot(self.toDot())
+  gum.MarkovBlanket._repr_html_ = lambda self: getDot(self.toDot())
